### ====================================================================
###  @Awk-file{
###     author          = "Nelson H. F. Beebe",
###     version         = "2.02",
###     date            = "23 June 1998",
###     time            = "18:48:02 MDT",
###     filename        = "man2html.awk",
###     address         = "Center for Scientific Computing
###                        University of Utah
###                        Department of Mathematics, 322 INSCC
###                        155 S 1400 E RM 233
###                        Salt Lake City, UT 84112-0090
###                        USA",
###     telephone       = "+1 801 581 5254",
###     FAX             = "+1 801 585 1640, +1 801 581 4148",
###     URL             = "http://www.math.utah.edu/~beebe",
###     checksum        = "16058 1872 5369 41413",
###     email           = "beebe@math.utah.edu, beebe@acm.org,
###                        beebe@ieee.org (Internet)",
###     codetable       = "ISO/ASCII",
###     keywords        = "HTML, nroff, troff, UNIX manual page",
###     supported       = "yes",
###     docstring       = "This program converts UNIX manual pages
###                        in nroff/troff markup to strictly-conformant
###                        HTML.
###
###                        Usage:
###                        	nawk -f man2html.awk \
###                                [-v BASENAME=basename] [-v HTML=level] \
###                                manpage-file >html-file
###
###                        If a BASENAME option is given, then the
###                        output file will be split into parts
###                        consisting of a root file containing a header
###                        and table of contents (written on stdout) and
###                        section files named basename-nn.html, where
###                        nn is a section number 01, 02, ....
###                        Normally, basename, manpage-filename, and
###                        html-filename will have the same name after
###                        any file extension is dropped.  This option
###                        permits large manual page files to be split
###                        into smaller parts that load faster over the
###                        World-Wide Web, although with the possibly
###                        significant disadvantage that the reader can
###                        no longer search the entire document with a
###                        single command.  A human needs to make that
###                        decision, so splitting is not automatically
###                        predicated on input file size.
###
###                        The second option, HTML=level, selects the
###                        HTML grammar level.  The default is HTML=2,
###                        since that level is implemented by most
###                        browsers.  The level choices are: 0, 1, 2,
###                        2-strict, 3, 3-strict, 3.2, 4, 4-loose,
###                        Cougar, Mosaic, or Netscape.
###
###                        Both options MUST be prefixed with the -v
###                        switch, which causes them to be available
###                        at startup time, before any files are read.
###
###                        Although it is possible to run this program
###                        with input provided on stdin, instead of from
###                        a named file, doing so will lose information
###                        in the header comments that record the name
###                        of the original file.
###
###                        This program is normally run via a shell
###                        wrapper that offers an option for setting the
###                        output file name.  It has been used to
###                        successfully convert entire man-page
###                        collections on several UNIX systems to HTML
###                        form for convenient World-Wide Web browser
###                        access.
###
###                        Of those nroff/troff commands defined in the
###                        -man format used for UNIX manual pages, only
###                        the most commonly-used ones are supported;
###                        unrecognized ones will be warned about, and
###                        preserved as HTML comments in the output.
###
###                        UNIX man pages tend to be written in a
###                        highly-stylized fashion that we apply
###                        heuristics to in order to recover high-level
###                        HTML structure from low-level nroff/troff
###                        markup.  Deviations from conventional
###                        man-page writing practice will likely result
###                        in less-than-perfect translation to HTML.
###
###                        Although there are several other `man2html'
###                        translators available on the Internet, this
###                        one is entirely of my own authorship, with no
###                        code borrowing from anywhere else.
###
###                        The checksum field above contains a CRC-16
###                        checksum as the first value, followed by the
###                        equivalent of the standard UNIX wc (word
###                        count) utility output of lines, words, and
###                        characters.  This is produced by Robert
###                        Solovay's checksum utility.",
###  }
### ====================================================================

BEGIN 		{ initialize() }

/^[.]ie +t +[.]ds/ { getline }	# fall through: next line should be .el

/^[.]el +.ds/	{ define($3); next }

/^[.']\\"/	{ cmd_comment($0); next } # save comments

/^[.]if +n *\\\{/ { cmd_comment($0); next }

/^[.]if +t *\\\{/ { cmd_comment_block($0); next } # convert troff directives to comments

/^ *\\\}/	{ cmd_comment($0); next }

/^[.]if +t/	{ cmd_comment($0); next } # convert troff directives to comments

/^[.]if +n +[.]ds/ { define($4); next }

/^[.]if +n +[.]ti/ { cmd_comment($0); next } # convert nroff spacing directives to comments

/^[.]ie +n +[.]ds/ { define($4); next }

/^[.]if +n +[.][A-Za-z]/	{ sub(/^[.]if +n +/,"") } # fall through

# Version 2.02: add 61 new troff commands found in Sun Solaris 2.6
# /usr/man/man*/* files (615,439 lines of data) that were omitted in
# previous versions of man2html.  Most still are handled by
# cmd_unknown(), but there is special code for .P, .SB, and .SM.

/^[.]ad/	{ cmd_ad(); next }

/^[.]AS/	{ cmd_AS(); next }

/^[.]as/	{ cmd_as(); next }

/^[.]B /	{ cmd_B(); next }

/^[.]bd/	{ cmd_bd(); next }

/^[.]BI /	{ cmd_XY("B","I"); next }

/^[.]bp/	{ cmd_bp(); next }

/^[.]BR /	{ cmd_XY("B","R"); next }

/^[.]br/	{ cmd_br(); next }

/^[.]BS/	{ cmd_BS(); next }

/^[.]ce[ 0-9]*$/ { cmd_ce(); next }

/^[.]CS/	{ cmd_CS(); next }

/^[.]cs/	{ cmd_cs(); next }

/^[.]de/	{ cmd_de(); next }

/^[.]DG/	{ cmd_DG(); next }

/^[.]ds/	{ cmd_ds(); next }

/^[.]DT/	{ cmd_DT(); next }

/^[.]dt/	{ cmd_dt(); next }

/^[.]dt/	{ cmd_dt(); next }

/^[.]EN/	{ cmd_EN(); next }

/^[.]EQ/	{ cmd_EQ(); next }

/^[.]ff/	{ cmd_ff(); next }

/^[.]fi/	{ cmd_fi(); next }

/^[.]ft/	{ cmd_ft(); next }

/^[.]GG/	{ cmd_GG(); next }

/^[.]gg/	{ cmd_gg(); next }

/^[.]HC/	{ cmd_HC(); next }

/^[.]hc/	{ cmd_hc(); next }

/^[.]hh/	{ cmd_hh(); next }

/^[.]HP/	{ cmd_HP(); next }

/^[.]ht/	{ cmd_ht(); next }

/^[.]hw/	{ cmd_hw(); next }

/^[.]hy/	{ cmd_hy(); next }

/^[.]I /	{ cmd_I(); next }

/^[.]IB /	{ cmd_XY("I","B"); next }

/^[.]ie/	{ cmd_ie(); next }

/^[.]IG/	{ cmd_IG(); next }

/^[.]ig/	{ cmd_ig(); next }

/^[.]in/	{ cmd_in(); next }

/^[.]IP/	{ cmd_IP(); next }

/^[.]Ip/	{ cmd_Ip(); next }

/^[.]IR /	{ cmd_XY("I","R"); next }

/^[.]IX /	{ cmd_IX(); next }

/^[.]jj/	{ cmd_jj(); next }

/^[.]ll/	{ cmd_ll(); next }

/^[.]LP/	{ cmd_LP(); next }

/^[.]na/	{ cmd_na(); next }

/^[.]ne/	{ cmd_ne(); next }

/^[.]nf/	{ cmd_nf(); next }

/^[.]NG/	{ cmd_NG(); next }

/^[.]nh/	{ cmd_nh(); next }

/^[.]No/	{ cmd_No(); next }

/^[.]nr/	{ cmd_nr(); next }

/^[.]ns/	{ cmd_ns(); next }

/^[.]P[^A-Za-z]/	{ cmd_P(); next }

/^[.]PD/	{ cmd_PD(); next }

/^[.]PE/	{ cmd_PE(); next }

/^[.]PO/	{ cmd_PO(); next }

/^[.]PP/	{ cmd_PP(); next }

/^[.]PS/	{ cmd_PS(); next }

/^[.]ps/	{ cmd_ps(); next }

/^[.]R /	{ cmd_R(); next }

/^[.]RB /	{ cmd_XY("R","B"); next }

/^[.]RE/	{ cmd_RE(); next }

/^[.]RF/	{ cmd_RF(); next }

/^[.]RI /	{ cmd_XY("R","I"); next }

/^[.]RS/	{ cmd_RS(); next }

/^[.]S[^A-Za-z]/	{ cmd_S(); next }

/^[.]SB/	{ cmd_SB(); next }

/^[.]SH/	{ cmd_SH(); next }

/^[.]SM/	{ cmd_SM(); next }

/^[.]so/	{ cmd_so(); next }

/^[.]sp/	{ cmd_sp(); next }

/^[.]SS/	{ cmd_SS(); next }

/^[.]ss/	{ cmd_ss(); next }

/^[.]T[^A-Za-z]/	{ cmd_T(); next }

/^[.]ta/	{ cmd_ta(); next }

/^[.]TE/	{ cmd_TE(); next }

/^[.]TG/	{ cmd_TG(); next }

/^[.]TH/	{ cmd_TH(); next }

/^[.]ti/	{ cmd_ti(); next }

/^[.]TP/	{ cmd_TP(); next }

/^[.]tr/	{ cmd_tr(); next }

/^[.]TS/	{ cmd_TS(); next }

/^[.]TX/	{ cmd_TX(); next }

/^[.]TZ/	{ cmd_TZ(); next }

/^[.]UC/	{ cmd_UC(); next }

/^[.]vs/	{ cmd_vs(); next }

/^[.]YY/	{ cmd_YY(); next }

/^[.]ZZ/	{ cmd_ZZ(); next }

/^[.]zZ/	{ cmd_zZ(); next }

/^[.]zz/	{ cmd_zz(); next }

/^[.][BIR]$/	{ cmd_BIR(); next }

/^[.][A-Za-z]/	{ cmd_unknown(); next }

		{ print_line(strtohtml($0)) }

END		{ terminate() }


# The anchor() function is adapted from my bibtex-to-html.awk file

function anchor(s,type,pattern,offset,prefix,save_label, name,rstart,rlength,save)
{
    # Add anchors <A type="....">...</A> around text in s matching
    # pattern.  A non-zero offset discards that many characters from
    # the start of the match, allowing the pattern to contain leading
    # context which goes outside the anchored region.  The prefix is
    # attached to the start of the matched string, inside the value
    # quotes in the anchor.

    if (match(s,pattern))
    {
	rstart = RSTART		# need private copies of these globals because
	rlength = RLENGTH	# recursion will change them

	rstart += offset	# adjust by offset to discard leading
	rlength -= offset	# context in pattern

	name = substr(s,rstart,rlength)
	sub(/ +at +/,"@",name)	# reduce "user at host" to "user@host"

	s = substr(s,1,rstart-1) \
	    "<A " type "=\"" prefix name "\">" \
	    ((type == "NAME") ? "<STRONG>" : "") \
	    substr(s,rstart,rlength) \
	    ((type == "NAME") ? "</STRONG>" : "") \
	    "</A>" \
	    anchor(substr(s,rstart+rlength),type,pattern,offset,prefix,save)
    }
    return (s)
}


function begin_document(outfile, save_TMPFILE)
{
    save_TMPFILE = TMPFILE	# print_line() uses global TMPFILE internally,
    TMPFILE = outfile		# so save it, and then restore before return.
    print_line("<!-- Warning: Do NOT edit this file. -->")
    print_line("<!-- It was created automatically by man2html.awk " VERSION " on " DATE " -->")
    print_line("<!-- from the file " strtohtml(FILENAME) " at " HOSTNAME " -->")
    if (BASENAME != "")		# then splitting output into multiple files
    {
	if (H1 > 0)
	    print_line("<!-- This is sectional file " H1 " of " H1_MAX \
		" file" ((H1_MAX == 1) ? "" : "s") " that accompan" \
		((H1_MAX == 1) ? "ies" : "y") " the root file. -->")
    }
    print_line("")
    print_line(Doctype[HTML])	# NB: initialize() ensures that we have this element!
    print_line("")
    TMPFILE = save_TMPFILE
}


function begin_toc()
{
    print_toc("<H1>")
    print_toc(TITLE)
    print_toc("</H1>")
    print_toc("<H2>")
    print_toc("Table of contents")
    print_toc("</H2>")
    print_toc("<UL>")
    In_TOC_Item = 0
}


function cmd_ad()
{				# .ad: turn on adjust (flush-left-and-right justification)
    cmd_comment($0)		# no HTML equivalent
}


function cmd_AS()
{
    cmd_unknown()
}


function cmd_B( s)
{
    end_font()
    if (match($0,/^[.]B *\"/))
    {
	s = substr($0,RSTART+RLENGTH)
	gsub(/[" ]*$/,"",s)
	print_line("<STRONG>" strtohtml(s) "</STRONG>")
    }
    else
	print_line("<STRONG>" strtohtml($2) "</STRONG>")
}


function cmd_BIR()
{
    end_font()
    print_line(strtohtml("\\f" substr($0,2,1))) # Remap .B into \fB etc
}


function cmd_BS()
{
    cmd_unknown()
}


function cmd_CS()
{
    cmd_unknown()
}


function cmd_DG()
{
    cmd_unknown()
}


function cmd_DT()
{
    # NYI: set tabs every 0.5inch from 0.0inch to 6.5inch
    cmd_unknown()
}


function cmd_EN()
{
    cmd_unknown()
}


function cmd_EQ()
{
    cmd_unknown()
}


function cmd_GG()
{
    cmd_unknown()
}


function cmd_HC()
{
    cmd_unknown()
}


function cmd_HP()
{
    # NYI: hanging indent
    cmd_unknown()
}


function cmd_I( s)
{
    end_font()
    if (match($0,/^[.]I *\"/))
    {
	s = substr($0,RSTART+RLENGTH)
	gsub(/[" ]*$/,"",s)
	print_line("<EM>" strtohtml(s) "</EM>")
    }
    else
	print_line("<EM>" strtohtml($2) "</EM>")

}


function cmd_IG()
{
    cmd_unknown()
}


function cmd_IP()
{
    end_font()
    PP++
    print_line("<P>")
}


function cmd_IX()
{
    # .IX index entry lines are simply discarded
    while (match($0,/\\$/) && (getline > 0))
	;			# discard continuation lines
}


function cmd_Ip()
{
    cmd_unknown()
}


function cmd_LP()
{
    end_font()
    PP++
    print_line("<P>")
}


function cmd_NG()
{
    cmd_unknown()
}


function cmd_No()
{
    cmd_unknown()
}


function cmd_P()
{
    cmd_PP($0)			# paragraph
}


function cmd_PD()
{
    # NYI: paragraph distance
    cmd_unknown()
}


function cmd_PE()
{
    cmd_unknown()
}


function cmd_PO()
{
    cmd_unknown()
}


function cmd_PP()
{
    end_font()
    PP++

    if (In_PRE)			# <P> tags are illegal in <PRE>...</PRE> environments
	print_line("")
    else
	print_line("<P>")

    end_TP()
}


function cmd_PS()
{
    cmd_unknown()
}


function cmd_R( s)
{
    end_font()
    if (match($0,/^[.]R *\"/))
    {
	s = substr($0,RSTART+RLENGTH)
	gsub(/[" ]*$/,"",s)
	print_line(strtohtml(s))
    }
    else
	print_line(strtohtml($2))
}


function cmd_RE()
{
    end_font()
    if (In_PRE)			# should not happen, but some man pages
	cmd_fi()		# are irregular
    while (List_Level > RSE_List_Level[RSE_Level])
	end_TP()
    if (RSE_Level > 0)
	RSE_Level--
    print_line("</BLOCKQUOTE>")
}


function cmd_RF()
{
    cmd_unknown()
}


function cmd_RS()
{
    end_font()
    RSE_List_Level[++RSE_Level] = List_Level
    List_Level++		# new .TP level too
    print_line("<BLOCKQUOTE>")
}


function cmd_S()
{
    cmd_unknown()
}


function cmd_SB( s)
{
    # small and boldface
    end_font()
    if (match($0,/^[.]SB *\"/))
    {
	s = substr($0,RSTART+RLENGTH)
	gsub(/[" ]*$/,"",s)
	print_line("<STRONG>" strtohtml("\\s-1" s "\\s0") "</STRONG>")
    }
    else
	print_line("<STRONG>" strtohtml("\\s-1" $2 "\\s0") "</STRONG>")
}


function cmd_SH( s)
{				# section heading
    cmd_SH_SS("H1")
}


function cmd_SH_SS(tag, s)
{				# [sub]section heading
    if (!TH_seen)		# should not happen, but some man pages are
	cmd_TH()		# irregular
    end_font()
    while (RSE_Level > 0)
	end_RS()
    while (List_Level > 0)
	end_TP()
    if (tag == "H1")
    {
	H1++
	if (H1 == 1)
	    begin_toc()
	if (H2 > 0)
	{
	    print_toc("</LI>")
	    print_toc("</UL>")
	}
	H2 = 0
	if (H1 > 1)
	    print_line("<HR>")	# a separating horizontal rule is a nice touch
    }
    else if (tag == "H2")
	H2++
    s = substr($0,5)
    sub(/^ *\"/,"",s)
    sub(/\" *$/,"",s)
    SH_SS_title = s		# save for later test
    s = strtohtml(s)

    if (tag == "H1")
	H1_history[H1] = s	# save for print_navigation_aid()

    SH_SS_count = "." H1
    if (H2 > 0)
	SH_SS_count = SH_SS_count "." H2

    print_line("<" tag ">")
    print_line("<A NAME=\"HDR" SH_SS_count "\">")
    print_line(s)
    print_line("</A>")
    print_line("</" tag ">")

    if (In_TOC_Item && (H2 != 1))
	print_toc("</LI>")
    if (H2 == 1)
	print_toc("<UL>")
    In_TOC_Item = 1
    print_toc("<LI>")
    print_toc("<A HREF=\"" make_filename(H1) "#HDR" SH_SS_count "\">")
    print_toc(s)
    print_toc("</A>")
}


function cmd_SM( s)
{
    # small
    end_font()
    if (match($0,/^[.]SM *\"/))
    {
	s = substr($0,RSTART+RLENGTH)
	gsub(/[" ]*$/,"",s)
	print_line(strtohtml("\\s-1" s "\\s0"))
    }
    else
	print_line(strtohtml("\\s-1" $2 "\\s0"))
}


function cmd_SS( s)
{				# subsection heading
    cmd_SH_SS("H2")
}


function cmd_T()
{
    cmd_unknown()
}


function cmd_TE()
{
    cmd_unknown()
}


function cmd_TG()
{
    cmd_unknown()
}


function cmd_TH()
{
    end_font()
    TITLE = $0
    while (TITLE ~ /\\$/)
    {
	getline
	TITLE = substr(TITLE,1,length(TITLE)-1) $0
    }
    TITLE = strtohtml(substr(TITLE,4))

    print_header(TMPFILE)
    TH_seen = 1
}


function cmd_TP()
{
    end_font()
    getline		# this is the item label, usually "\(bu" or ".B ..."
    if (Item_Count[List_Level] == 0) # then first item of new list
    {
	List_Level++
	Item_Count[List_Level] = 0
        if ($0 == "\\(bu")
	{
	    List_Name[List_Level] = "UL"
	    List_Item[List_Level] = "LI"
	}
	else
	{
	    List_Name[List_Level] = "DL"
	    List_Item[List_Level] = "DT"
	}
	if (Item_Count[List_Level] == 0)
	    print_line("<" List_Name[List_Level] ">")
    }
    Item_Count[List_Level]++
    if ((SH_SS_title == "OPTIONS") && \
	(List_Level == 1) && \
	(List_Name[List_Level] == "DL"))
    {				# create TOC entries for command-line options
	if (Item_Count[List_Level] == 1)
	    print_toc("<UL>")
	else
	    print_toc("</LI>")
	SH_OPTION_count = "." H1 "." Item_Count[List_Level]
	print_toc("<LI>")
	print_toc("<A HREF=\"" make_filename(H1) "#OPT" SH_OPTION_count "\">")
	print_toc(strtohtml($2))
	print_toc("</A>")
    }
    if (List_Name[List_Level] == "DL")
    {
	if (Item_Count[List_Level] > 1)
	    print_line("</DD>")
	print_line("<DT>")
	if ((SH_SS_title == "OPTIONS") && \
	    (List_Level == 1) && \
	    (List_Name[List_Level] == "DL"))
	    print_line("<A NAME=\"OPT" SH_OPTION_count "\">")
	if ($0 ~ /^[.]B /)
	    cmd_B()
	else if ($0 ~ /^[.]I /)
	    cmd_I()
	else if ($0 ~ /^[.]R /)
	    cmd_R()
	else if ($0 ~ /^[.]BR/)
	    cmd_XY("B","R")
	else if ($0 ~ /^[.]BI/)
	    cmd_XY("B","I")
	else if ($0 ~ /^[.]IB/)
	    cmd_XY("I","B")
	else if ($0 ~ /^[.]IR/)
	    cmd_XY("I","R")
	else if ($0 ~ /^[.]RB/)
	    cmd_XY("R","B")
	else if ($0 ~ /^[.]RI/)
	    cmd_XY("R","I")
	else if ($0 ~ /^[.]SB/)
	    cmd_SB()
	else if ($0 ~ /^[.]SM/)
	    cmd_SB()
	else
	    print_line(strtohtml($0))
	if ((SH_SS_title == "OPTIONS") && (List_Level == 1))
	    print_line("</A>")
	end_font()
	if (In_PRE)		# should not happen, but some man pages
	    cmd_fi()		# are irregular
	print_line("</DT>")
	print_line("<DD>")
    }
    else			# must be <UL> <LI> ... </LI> </UL> type list
    {
	if (Item_Count[List_Level] > 1)
	    print_line("</LI>")
	print_line("<LI>")
    }
}


function cmd_TS( tbl_nroff_cmd)
{
    # Copy the table to a temporary file
    print $0 >TBLFILE
    while (getline > 0)
    {
	print $0 >TBLFILE
	if ($0 ~ /^[.]TE/)	# then end of table found
	    break
    }
    close (TBLFILE)

    # Run tbl, nroff, and col to convert the table to
    # formatted text, and include it as a preformatted
    # environment.
    tbl_nroff_cmd = "tbl " TBLFILE " | nroff -man | col -b"

    print_line("<PRE>")
    while ((tbl_nroff_cmd | getline) > 0)
	print_line(strtohtml($0))
    print_line("</PRE>")
    close (tbl_nroff_cmd)
    delete_file(TBLFILE)
}


function cmd_TX()
{
    cmd_unknown()
}


function cmd_TZ()
{
    # resolve title reference
    cmd_unknown()
}


function cmd_UC()
{
    cmd_unknown()
}


function cmd_XY(x,y, font,k,s)
{
    end_font()
    protect_quoted_args()
    for (k = 2; k <= NF; ++k)
    {
	font = Font_Map[(k % 2) ? y : x]
	s = strtohtml(unprotect_quoted_arg($k))
	if (s != "")		# optimization: ignore empty strings
	    printf("%s%s%s", html_font_begintag(font), s, \
		html_font_endtag(font)) > TMPFILE
    }
    print_line("")
}


function cmd_YY()
{
    cmd_unknown()
}


function cmd_ZZ()
{
    cmd_unknown()
}


function cmd_as()
{
    cmd_unknown()
}


function cmd_bd()
{
    cmd_unknown()
}


function cmd_bp()
{
    cmd_unknown()
}


function cmd_br()
{
    cmd_PP()
}


function cmd_ce( k,n)
{
    # .ce nnn: turn on centering for next nnn lines (nnn = 0 turns it off)

    n = $2
    cmd_comment($0)
    if (n > 0)
    {
	# The HTML 3.2 grammar supports <CENTER> ... </CENTER> as a
	# shorthand for the more general <DIV ALIGN=CENTER> ... </DIV>
	# (CENTER can be replaced by LEFT or RIGHT).  However, except
	# for amaya (W3C's testbed for HTML 3.2), none of the current
	# browsers support DIV. grail, hotjava, netscape all recognize
	# CENTER. arena, chimera, lynx, and xmosaic do not recognize it
	# either.  It is in the 4.0 loose grammar, but absent from the
	# 4.0 strict grammar
	if (have_feature("<CENTER>"))
	    print_line("<CENTER>")
	else			# record the request as a comment
	    print_line("<!-- <CENTER> -->")
	for (k = 1; k <= n; ++k)
	{
	    getline
	    print_line(strtohtml($0) "<BR>")
	}
	if (have_feature("<CENTER>"))
	    print_line("</CENTER>")
	else			# record the request as a comment
	    print_line("<!-- </CENTER> -->")
    }
}


function cmd_comment(s)
{
    In_Comment = 1
    sub(/^[.']\\"/,"",s)	# remove troff comment prefix: it confuses html-pretty
    print_line("<!-- " strtohtml(s) " -->")
    In_Comment = 0
}


function cmd_comment_block(s)
{
    cmd_comment(s)

    In_Comment = 1
    while (getline s > 0)
    {
	cmd_comment(s)
	if (s ~ /^ *\\\}/)
	    break		# found end of block
    }
    In_Comment = 0
}


function cmd_cs()
{
    cmd_unknown()
}


function cmd_de()
{
    cmd_unknown()
}


function cmd_ds()
{
    cmd_unknown()
}


function cmd_dt()
{
    cmd_unknown()
}


function cmd_el()
{
    cmd_unknown()
}


function cmd_ff()
{
    cmd_unknown()
}


function cmd_fi()
{
    end_font()
    if (In_PRE)
    {
	print_line("</PRE>")
	In_PRE = 0
    }
    else
	cmd_comment($0)
}


function cmd_ft()
{
    cmd_unknown()
}


function cmd_gg()
{
    cmd_unknown()
}


function cmd_hc()
{
    cmd_unknown()
}


function cmd_hh()
{
    cmd_unknown()
}


function cmd_ht()
{				# .ht nnn: hyphenation threshold
    cmd_comment($0)
}


function cmd_hw()
{				# .hw word-hyph-en-a-tion ex-cep-tions
    cmd_comment($0)
}


function cmd_hy()
{		# .hy and .hy nnn: turn hyphenation on (or off, when nnn=0)
    cmd_comment($0)
}


function cmd_ie()
{
    cmd_unknown()
}


function cmd_ig()
{
    cmd_unknown()
}


function cmd_in()
{
    cmd_unknown()
}


function cmd_jj()
{
    cmd_unknown()
}


function cmd_ll()
{
    cmd_unknown()
}


function cmd_na()
{	# .na: no adjust: turn off flush-left-and-right justification, producing ragged-right
    cmd_comment($0)		# no HTML equivalent
}


function cmd_ne()
{				# .ne dimen: need dimen vertical space before end of page
				# otherwise, force a page break (e.g. to prevent page
				# breaks after headings)
    cmd_comment($0)
}


function cmd_nf()
{
    end_font()
    if (In_PRE)
	cmd_comment($0)
    else
    {
	print_line("<PRE>")
	In_PRE = 1
    }
}


function cmd_nh()
{				# .nh: turn off hyphenation
    cmd_comment($0)
}


function cmd_nr()
{
    cmd_unknown()
}


function cmd_ns()
{
    cmd_unknown()
}


function cmd_ps()
{
    cmd_unknown()
}


function cmd_so()
{
    cmd_unknown()
}


function cmd_sp()
{				# .sp nnn: vertical space
    cmd_comment($0)		# no sensible HTML equivalent
}


function cmd_ss()
{
    cmd_unknown()
}


function cmd_ta()
{
    cmd_unknown()
}


function cmd_ti()
{
    cmd_unknown()
}


function cmd_tr()
{
    cmd_unknown()
}


function cmd_unknown()
{
    end_font()
    warning("Unrecognized nroff/troff command in [" $0 "] changed to comment")
    cmd_comment($0)
}


function cmd_vs()
{
    cmd_unknown()
}


function cmd_zZ()
{
    cmd_unknown()
}


function cmd_zz()
{
    cmd_unknown()
}


function define(name, regexp)
{
    # Typical values:
    # .if n .ds Bi BibTeX
    # .el .ds Bi BibTeX
    # Macro used as \*(Bi, but stored as a regexp
    regexp = "\\\\\\*\\(" name
    Macro[regexp] = substr($0,index($0,name)+3)
}


function delete_file(s)
{
    system("/bin/rm -f " s)
}


function end_document(outfile, save_TMPFILE)
{
    end_font()
    while (RSE_Level > 0)
	end_RS()
    while (List_Level > 0)
	end_TP()

    # write out the end boilerplate (</BODY></HTML>) and close outfile
    save_TMPFILE = TMPFILE	# print_line() uses global TMPFILE internally,
    TMPFILE = outfile		# so save it, and then restore before return.
    print_navigation_aid()
    print_line("</BODY>")
    print_line("</HTML>")
    if (TMPFILE != STDOUT)	# cannot close STDOUT in some awk implementations
	close (TMPFILE)
    TMPFILE = save_TMPFILE
}


function end_font()
{
    for (; Font_Level > 0; Font_Level--)
	print_line(html_font_endtag(HTML_Font_Name[Font_Level]))
}


function end_RS()
{
    # This function currently does nothing.
}


function end_toc()
{
    print_toc("</LI>")
    print_toc("</UL>")
    print_toc("<HR>")
    close (TOCFILE)
}


function end_TP()
{
    if (Item_Count[List_Level] > 0)
    {
	if (List_Name[List_Level] == "DL")
	{
	    print_line("</DD>")
	    print_line("</DL>")
	    if ((SH_SS_title == "OPTIONS") && (List_Level == 1))
	    {
		print_toc("</LI>")
		print_toc("</UL>")
	    }
	}
	else
	{
	    print_line("</LI>")
	    print_line("</UL>")
	}
    }
    Item_Count[List_Level] = 0
    if (List_Level > 0)
	List_Level--
}


function font_sub(s, tag)
{
    while (match(s,/\\f[BCIPRST]/))
    {
	if (substr(s,RSTART+2,1) == "P") # revert to previous font
	{
	    tag = html_font_endtag(HTML_Font_Name[Font_Level])
	    if (Font_Level > 0)
		Font_Level--
	}
	else			# set explicit font
	{
	    Font_Level++
	    HTML_Font_Name[Font_Level] = Font_Map[substr(s,RSTART+2,1)]
	    tag = html_font_begintag(HTML_Font_Name[Font_Level])
	    # Handle ...\fB...\fR... style by ending previous font
	    if (Font_Level > 1)
	    {
		tag = html_font_endtag(HTML_Font_Name[Font_Level-1]) tag
		HTML_Font_Name[Font_Level-1] = HTML_Font_Name[Font_Level]
		Font_Level--
	    }
	}
	s = substr(s,1,RSTART-1) tag substr(s,RSTART+3)
    }
    while (match(s,/\\s[-+][0-9][^\\]*\\s0/))
    {				# handle only simple paired inline size changes for now
	if (substr(s,RSTART+2,1) == "-") # \s-nxxx\s0 -> <SMALL>xxx</SMALL>
	    s = substr(s,1,RSTART-1) \
		(have_feature("<SMALL>") ? "<SMALL>" : "<!--SMALL-->") \
		substr(s,RSTART+4,RLENGTH-7) \
		(have_feature("<SMALL>") ? "</SMALL>" : "<!--/SMALL-->") \
		substr(s,RSTART+RLENGTH)
	else			# \s+nxxx\s0 -> <BIG>xxx</BIG>
	    s = substr(s,1,RSTART-1) \
		(have_feature("<BIG>") ? "<BIG>" : "<!--BIG-->") \
		substr(s,RSTART+4,RLENGTH-7) \
		(have_feature("<BIG>") ? "</BIG>" : "<!--/BIG-->") \
		substr(s,RSTART+RLENGTH)
    }
    return (s)
}


function have_feature(feature)
{
    if (feature in Feature_List)
	return (index(Feature_List[feature],(" " HTML " ")) > 0)
    else
    {
	warning("please report this internal error: feature [" feature \
	    "] is not in Feature_List[]")
	return (0)
    }
}


function html_font_begintag(name)
{
    if (name == "")
	return ""
    else
	return "<" name ">"
}


function html_font_endtag(name)
{
    if (name == "")
	return ""
    else
	return "</" name ">"
}


function initialize( procid)
{
    # Change these two lines whenever the program is modified
    VERSION_NUMBER = "2.02"
    VERSION_DATE = "[23-Jun-1998]"

    VERSION = "Version " VERSION_NUMBER " " VERSION_DATE

    STDOUT = "/dev/stdout"	# this appears to be awk-version-dependent, and
				# system-dependent, but actually is just a magic flag

    "echo $LOGNAME" | getline LOGNAME
    close ("echo $LOGNAME")

    "hostname" | getline HOSTNAME
    close ("hostname")

    "date" | getline DATE
    close ("date")

    # To facilite distinguishing between grammar-level-dependent
    # features, and avoid the need to change if-statements that test
    # for them, from version 1.07, the function have_feature() uses
    # the global Feature_List[] array.  The grammar levels in the
    # string values MUST be surrounded by space.  The global Doctype[]
    # array maps grammar levels to DOCTYPE strings.

    Doctype["0"]	= "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML Level 0//EN\">"
    Doctype["1"]	= "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML Level 1//EN\">"
    Doctype["2"]	= "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML Level 2//EN\">"
    Doctype["2-strict"]	= "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML Strict Level 2//EN\">"
    Doctype["3"]	= "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML Level 3//EN\">"
    Doctype["3-strict"]	= "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML Strict Level 3//EN\">"
    Doctype["3.2"]	= "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">"
    Doctype["4"]	= "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\">"
    Doctype["4-loose"]	= "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">"
    Doctype["Cougar"]	= "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML Experimental 19960712//EN\">"
    Doctype["Mosaic"]	= "<!DOCTYPE HTML PUBLIC \"-//Netscape Comm. Corp.//DTD HTML//EN\">"
    Doctype["Netscape"]	= "<!DOCTYPE HTML PUBLIC \"-//Netscape Comm. Corp.//DTD HTML//EN\">"

    Feature_List["<BIG>"]	= " 3 3.2 4-loose Cougar "
    Feature_List["<CENTER>"]	= " 3.2 4-loose Cougar Mosaic Netscape "
    Feature_List["<SMALL>"]	= " 3 3.2 4-loose Cougar "

    # This list is derived from the HTML *.dtd files; none are currently
    # needed in man2html, but might be in the future, so we record them
    # here now.
    Feature_List["&amp;"]	= " Cougar 3.2-draft 3.2 "
    Feature_List["&copy;"]	= " Cougar 3.2-draft 3 "
    Feature_List["&emsp;"]	= " Cougar 3 "
    Feature_List["&ensp;"]	= " Cougar 3 "
    Feature_List["&gt;"]	= " Cougar 3.2-draft 3.2 "
    Feature_List["&lrm;"]	= " Cougar "
    Feature_List["&lt;"]	= " Cougar 3.2-draft 3.2 "
    Feature_List["&mdash;"]	= " Cougar 3 "
    Feature_List["&nbsp;"]	= " 2 Cougar Mosaic Netscape 3.2-draft 3 "
    Feature_List["&ndash;"]	= " Cougar 3 "
    Feature_List["&quot;"]	= " 2 Cougar Mosaic Netscape 3.2-draft "
    Feature_List["&reg;"]	= " Cougar 3.2-draft 3 "
    Feature_List["&rlm;"]	= " Cougar "
    Feature_List["&shy;"]	= " Cougar 3 "
    Feature_List["&thinsp;"]	= " Cougar "
    Feature_List["&trade;"]	= " Cougar 3 "
    Feature_List["&zwj;"]	= " Cougar "
    Feature_List["&zwnj;"]	= " Cougar "

    if (HTML == "")		# provide default level when not given on command line
	HTML = 2

    if (!(HTML in Doctype))
    {
        warning("Unsupported HTML level " HTML " requested: defaulting to HTML level 2")
	HTML = 2
    }

    Font_Map["B"] = "STRONG"
    Font_Map["C"] = "TT"
    Font_Map["I"] = "EM"
    Font_Map["R"] = ""
    Font_Map["S"] = ""		# cannot map symbol font yet
    Font_Map["T"] = "TT"

    Macro["\\\\e"] = "\\"
    Macro["\\\\0"] = have_feature("&nbsp;") ? "\\&nbsp;" : "\\&#160;"

    srand()			# seed random-number generator with time-of-day
    procid = int(2147483647 * rand()) # and get a random number in 0..2^31-1
    TOCFILE = "/tmp/man2html.toc." procid
    TBLFILE = "/tmp/man2html.tbl." procid
    TMPFILE = "/tmp/man2html.tmp." procid
    H1 = 0
    H2 = 0

    # It is curious that browsers can display a bullet, but there is no
    # HTML markup to represent it, and it is absent from the standard
    # ISO8859-1 fonts.  We choose general-currency-sign (&curren;) but
    # use its numeric code because many browsers do not recognize the
    # named entity.
    Macro["\\\\\\(bu"]	= "\\&#164;"

    Macro["\\\\\\(em"]	= "---"
    Macro["\\\\\\(en"]	= "--"

    # The following fragment for setting URL_xxx variables
    # is borrowed intact from my bibtex-to-html.awk file:
    #
    # According to Internet RFC 1614 (May 1994), a URL is
    # defined in the document T. Berners-Lee, ``Uniform
    # Resource Locators'', March 1993, available at URL
    # ftp://info.cern.ch/pub/ietf/url4.ps.  Unfortunately,
    # that address is no longer valid.  However, I was able to
    # track down pointers from http://www.w3.org/ to locate a
    # suitable description in Internet RFC 1630 (June 1994).

    # NB: We additionally disallow & in a URL because it is
    # needed in SGML entities "&name;".  We also disallow =
    # and | because these are commonly used in \path=...= and
    # \path|...| strings in BibTeX files.  These restrictions
    # could be removed if we went to the trouble of first
    # encoding these special characters in %xy hexadecimal
    # format, but they are rare enough that I am not going to
    # do so for now.  The worst that will happen from this
    # decision is that an occasional URL in a BibTeX file will
    # be missing a surrounding anchor.

    # Bug fix [24-Oct-1997]: Add < and > to the set of excluded
    # characters, to avoid incorrectly including SGML markup inside a
    # URL.  Before this fix, "\fChttp://www/\fP" got translated
    # incorrectly to
    #     <TT><A HREF="http://www/</TT>">http://www/</TT></A>
    # instead of the correct
    #     <TT><A HREF="http://www">http://www</A></TT>

    URL_PATTERN = "[A-Za-z]+://[^ \",&=|<>]+"
    URL_OFFSET = 0
    URL_PREFIX = ""
    URL_SAVE_LABEL = 0

    E_MAIL_PATTERN = "[A-Za-z0-9_-]+@[A-Za-z0-9-]+([.][A-Za-z0-9-]+)*"
    E_MAIL_OFFSET = 0
    E_MAIL_PREFIX = "mailto:"
    E_MAIL_SAVE_LABEL = 0

    begin_document(STDOUT)
}


function make_filename(H1_count, basename)
{
    if (BASENAME == "")		# then no splitting requested
	basename = ""		# and basename is an empty string
    else			# else construct a section-numbered filename
    {
	# Construct a filename for use in the HREF fields and output
	# filenames.  Different awk implementations use different names for
	# unnamed input files or pipes, but in practice, when output file
	# splitting is requested, we should always have a filename
	# available.
	basename = (BASENAME != "") ? \
			BASENAME : \
			((FILENAME == "-") || \
			 (FILENAME == " ") || \
			 (FILENAME == "")) ? "manpage" : FILENAME
	gsub(/[.][^.\/]*$/,"",basename) # strip extension
	if (H1_count > 0)
	    basename = sprintf("%s-%02d", basename, H1_count)
	basename = (basename ".html")
    }
    return (basename)
}


function make_navigation_aid(H1_offset, s)
{
    s = "    <A HREF=\"" \
	make_nth_filename((H1_offset == 0) ? 0 : (H1 + H1_offset)) \
	"\">"
    s = s "<STRONG>"

    if (H1_offset < 0)
	s = s "." H1_offset
    else if (H1_offset == 0)
	s = s "Top"
    else # H1_offset > 0
	s = s ".+" H1_offset

    if (H1_offset != 0)
    {
	s = s "["
	s = s ((((H1 + H1_offset) < 1) || (H1_MAX < (H1 + H1_offset))) ? \
	    "Top" : H1_history[H1 + H1_offset])
	s = s "]"a
    }

    s = s "</STRONG>"
    s = s "</A>"

    return (s)
}


function make_nth_filename(H1_count)
{
    # Return a filename corresponding to section H1_count.  If that
    # count is negative, or beyond the last section number, H1_MAX,
    # then return the root filename.   This is a convenience function
    # for print_navigation_aid().
    return (make_filename(((H1_count < 0) || (H1_MAX < H1_count)) ? 0 : H1_count))
}


function print_header(outfile, save_TMPFILE)
{
    save_TMPFILE = TMPFILE	# print_line() uses global TMPFILE internally,
    TMPFILE = outfile		# so save it, and then restore before return.
    print_line("<HTML>")
    print_line("<HEAD>")
    print_line("<TITLE>")
    print_line((BASENAME == "") ? \
	       TITLE : \
	       (TITLE ((H1 == 0) ? " [root]" : (" [section " H1 " of " H1_MAX "]"))))
    print_line("</TITLE>")
    print_line("<LINK REV=\"made\" HREF=\"mailto:" LOGNAME "@" HOSTNAME "\">")
    print_line("</HEAD>")
    print_line("")
    print_line("<BODY>")
    print_line("")
    if ((TITLE != "") && TH_seen)
    {
	print_line("<H1>")
	print_line((BASENAME == "") ? \
		   TITLE : \
		   (TITLE ((H1 == 0) ? " [root]" : (" [section " H1 " of " H1_MAX "]"))))
	print_line("</H1>")
	print_navigation_aid()
    }
    TMPFILE = save_TMPFILE
}


function print_line(s)
{
    if (TMPFILE == STDOUT)	# some awk implementations lack a name for stdout, sigh...
	print s
    else
	print s >TMPFILE
}


function print_navigation_aid()
{
    if ((BASENAME != "") && (H1 > 0))
    {
	    print_line("")
	    print_line("<P></P>") # workaround for html-pretty's supplying <P> after section headings
	    print_line(have_feature("<CENTER>") ? "<CENTER>" : "<!-- <CENTER> -->")
	    print_line("<P>")
	    if ((H1 - 3) >= 1)
	    {
		print_line(make_navigation_aid(-3))
		print_space_separator()
		print_space_separator()
	    }
	    if ((H1 - 2) >= 1)
	    {
		print_line(make_navigation_aid(-2))
		print_space_separator()
		print_space_separator()
	    }
	    if ((H1 - 1) >= 1)
	    {
		print_line(make_navigation_aid(-1))
		print_line("    <BR>")
	    }

	    print_line(make_navigation_aid(0))

	    if ((H1 + 1) <= H1_MAX)
	    {
		print_line("    <BR>")
		print_line(make_navigation_aid(+1))
	    }
	    if ((H1 + 2) <= H1_MAX)
	    {
		print_space_separator()
		print_line(make_navigation_aid(+2))
	    }
	    if ((H1 + 3) <= H1_MAX)
	    {
		print_space_separator()
		print_space_separator()
		print_line(make_navigation_aid(+3))
	    }
	    print_line("</P>")
	    print_line(have_feature("<CENTER>") ? "</CENTER>" : "<!-- </CENTER> -->")
	    print_line("")
	    print_line("<HR>")	# we do not want a rule at start of aid,
	      			# because there already is one at the
				# end of every section, except the top
    }
}


function print_space_separator()
{
    print_line(have_feature("&nbsp;") ? "    &nbsp;&nbsp;&nbsp;" : "    &#160;&#160;&#160;")
}


function print_toc(s)
{
    print s >TOCFILE
}


function protect_quoted_args( inside,k,s)
{
    if (index($0,"\"") == 0)
	return
    s = $0
    inside = 0
    for (k = 1; k <= length(s); ++k)
    {
	if (substr(s,k,1) == "\"")
	    inside = !inside
	else if (inside && (substr(s,k,1) == " "))
	    s = substr(s,1,k-1) "\177" substr(s,k+1)
    }
    $0 = s
}


function strtohtml(s, name)
{
    if (In_Comment)
    {
	gsub(/-/,"_",s)		# change hyphen to underscore to prevent
				# comment syntax confusion
	gsub(/</,"\\&lt;",s)	# protect two important special SGML
	gsub(/>/,"\\&gt;",s)	# characters
    }
    else
    {
	gsub(/\\$/,"",s)	# discard backslash-newline
	gsub(/\\-/,"-",s)	# show troff minus as ASCII minus
	gsub(/\\[&]/,"",s)	# remove no-op macros
	# gsub(/\\[|]/," ",s)	# change thin space to space
	gsub(/\\[|]/,"",s)	# delete thin space (nroff does too)

	gsub(/[&]/,"\\&amp;",s)	# protect 3 or 4

	gsub(/</,"\\&lt;",s)	# protect two important special SGML
	gsub(/>/,"\\&gt;",s)	# characters

	gsub(/\\ /,(have_feature("&nbsp;") ? "\\&nbsp;" : "\\&#160;"),s)
				# translate literal space
	if (have_feature("&shy;"))
	{
	    # NB: several browers fail to implement soft hyphen properly: they show
	    # it as an explicit hyphen when the word is not broken at end of line,
	    # instead of discarding it.  We translate it correctly, and hope that
	    # broken browsers eventually get fixed, sigh...
	    warning("soft hyphen (&shy;) entity is handled incorrectly by several browsers")
	    gsub(/\\%/,"\\&shy;",s) # discretionary hyphen -> soft hyphen
	}
	else
	    gsub(/\\%/,"",s)	# squeeze out discretionary hyphens

	gsub(/\"/,(have_feature("&quot;") ? "\\&quot;" : "\\&#34;"),s)

	for (name in Macro)	# substitute macro names
	    gsub(name,Macro[name],s)

	if (!In_Comment)
	    s = font_sub(s)

	gsub(/\\\\/,"\\",s)	# reduce troff doubled backslash to single HTML one

#	if (index(s,"\\") > 0)	# check for anything we missed
#	    warning("Possible unrecognized nroff/troff markup in [" s "]")

	s = anchor(s,"HREF",URL_PATTERN,URL_OFFSET,URL_PREFIX,URL_SAVE_LABEL)
	s = anchor(s,"HREF",E_MAIL_PATTERN,E_MAIL_OFFSET,E_MAIL_PREFIX, \
		   E_MAIL_SAVE_LABEL)
    }

    return (s)
}


function terminate( newfile,oldfile,x,y)
{
    H1_MAX = H1			# so we can generate "[part H1 of H1_MAX]" in TITLE

    end_document(TMPFILE)
    end_toc()

    H1 = 0			# prevent printing of navigation aid in this file

    while ((getline x < TMPFILE) > 0) # copy section 0 of TMPFILE to stdout
    {
        if (x == "<H1>")
	{
	    H1++
	    break
	}
	print x
    }

    while ((getline y < TOCFILE) > 0)
	print y
    close (TOCFILE)
    delete_file(TOCFILE)

    newfile = make_filename(H1)

    # copy the <H1> line to the start of a (possibly new) file
    if (newfile == "")		# single output file on stdout
	print x
    else			# output is split into multiple files
    {
	end_document(STDOUT)
	begin_document(newfile)
	print_header(newfile)
	print x >newfile
    }

    while ((getline x < TMPFILE) > 0)
    {
	if (x == "<H1>")
	{
	    oldfile = newfile
	    newfile = make_filename(H1 + 1)
	    if (oldfile != "")	# then must be splitting output
	    {
		end_document(oldfile)
		close(oldfile)
		H1++		# delayed increment because end_document() needs current value
		begin_document(newfile)
		print_header(newfile)
	    }
	    else
		H1++		# delayed increment because end_document() needs current value
	}
	if (newfile == "")	# single output file on stdout
	    print x
	else			# output is split into multiple files, currently, newfile
	    print x >newfile
    }
    if (newfile != "")
	close (newfile)
    close (TMPFILE)
    delete_file(TMPFILE)
}


function unprotect_quoted_arg(s)
{
    sub(/^"/,"",s)		# remove leading and
    sub(/"$/,"",s)		# trailing quotes and
    gsub(/\177/," ",s)		# restore spaces
    return (s)
}


function warning(message)
{
    print FILENAME ":" FNR ":%%" message >"/dev/stderr"
}
