.\" ====================================================================
.\"  @Troff-man-file{
.\"     author          = "Nelson H. F. Beebe",
.\"     version         = "2.02",
.\"     date            = "23 June 1998",
.\"     time            = "18:03:26 MDT",
.\"     filename        = "man2html.man",
.\"     address         = "Center for Scientific Computing
.\"                        University of Utah
.\"                        Department of Mathematics, 322 INSCC
.\"                        155 S 1400 E RM 233
.\"                        Salt Lake City, UT 84112-0090
.\"                        USA",
.\"     telephone       = "+1 801 581 5254",
.\"     FAX             = "+1 801 585 1640, +1 801 581 4148",
.\"     URL             = "http://www.math.utah.edu/~beebe",
.\"     checksum        = "59043 351 1406 9944",
.\"     email           = "beebe@math.utah.edu, beebe@acm.org,
.\"                        beebe@ieee.org (Internet)",
.\"     codetable       = "ISO/ASCII",
.\"     keywords        = "HTML, nroff, troff, UNIX manual page",
.\"     supported       = "yes",
.\"     docstring       = "This is the UNIX manual page documentation
.\"                        for man2html, a converter of UNIX manual page
.\"                        files to strictly-grammar-conforming HTML.
.\"
.\"                        The checksum field above contains a CRC-16
.\"                        checksum as the first value, followed by the
.\"                        equivalent of the standard UNIX wc (word
.\"                        count) utility output of lines, words, and
.\"                        characters.  This is produced by Robert
.\"                        Solovay's checksum utility.",
.\"  }
.\" ====================================================================
.TH MAN2HTML 1 "23 June 1998" "Version 2.02"
.\"=====================================================================
.SH NAME
man2html \- convert a UNIX manual page file from nroff/troff -man format to HTML
.\"=====================================================================
.SH SYNOPSIS
.na
.B man2html
[
.B \-check-html
]
[
.BI \-grammar-level " grammar"
]
.if n .ti +9n
[
.BI \-outdirectory " directoryname"
]
.if t .ti +\w'\fBman2html\fP\ 'u
[
.B \-prettyprint
]
.if n .ti +9n
[
.BI \-split-limit " filesize-in-bytes"
]
.if n .ti +9n
.if t .ti +\w'\fBman2html\fP\ 'u
.I input-manpage-file(s)
.ad
.\"=====================================================================
.SH DESCRIPTION
.B man2html
converts UNIX manual page files named on the
command line, from
.BR nroff (1)/ troff (1)
.I \-man
format to strictly-grammar-conforming HTML.
.PP
The output files have the same base name (or the
base name with a numeric suffix, if output HTML
file splitting is requested), but extension
.IR .html .
.PP
Although some vendors, such as Sun Microsystems,
provide clear documentation of how manual pages
should be written, many manual page authors ignore
those recommendations, and use arbitrary
.RB [ nt ] roff
markup to achieve the traditional appearance of
UNIX manual pages, without actually using the
standard
.I \-man
format commands.
.PP
.B man2html
works quite well on Sun manual pages, but may be
less successful on manual pages from other
sources.  In such a case, an alternative may be to
use T. A. Phelp's
.BR RosettaMan (1),
commonly installed as
.BR rman (1).
That program works on the
.I output
of
.BR nroff (1),
and attempts to guess manual page structure from
the horizontal and vertical spacing in order to
add HTML markup.  When vendor-provided manual
pages are available only in preformatted form, as
on IBM AIX and SGI IRIX systems,
.BR rman (1)
may be your only choice.  However, when
.B man2html
can be used successfully, it can often do a better
job than
.BR rman (1),
because it has a better understanding of the
document structure implied by
.RB [ nt ] roff
manual-page markup.
.\"=====================================================================
.SH OPTIONS
Command-line options may be abbreviated to any
unique prefix, and letter case is
.IR significant .
Options and files are processed in the order
found; thus, options affect only files that follow
them on the command line.
.\"-----------------------------------------------
.TP \w'\fB\-prettyprint\fP'u+3n
.B \-check-html
Check the output HTML for validity with a rigorous
SGML parser, such as
.BR html-check (1)
or
.BR html-ncheck (1).
.\"-----------------------------------------------
.TP
.BI \-grammar-level " grammar"
Specify a grammar level to select a suitable
.RS
.nf
\fC<!DOCTYPE HTML PUBLIC ".\|.\|.">\fP
.fi
.RE
.IP
declaration.  Acceptable values are:
.IR 0 ,
.IR 1 ,
.IR 2 ,
.IR 2-strict ,
.IR 3 ,
.IR 3-strict ,
.IR 3.2 ,
.IR 4 ,
.IR 4-loose ,
.IR Cougar ,
.IR Mosaic ,
and
.IR Netscape
[default:
.IR 2 ].
.\"-----------------------------------------------
.TP
.BI "\-outdirectory" " directoryname"
Provide an alternate directory into which output
HTML files are placed.  The default is the current
directory.
.\"-----------------------------------------------
.TP
.BI \-prettyprint
Prettyprint each output file with
.BR html-pretty (1).
Prettyprinting is done before any syntax check
requested by the
.B \-check-html
option.
.\"-----------------------------------------------
.TP
.BI \-split-limit " filesize-in-bytes"
Split the translated HTML from input files that
are larger than the specified size into multiple
output HTML files: a root file, and section files
named
.IR basename-nn.html ,
where
.I basename
is the manual page file name with directory path
and extension removed, and
.I nn
is a section number 01, 02, .\|.\|.
[default: no output splitting].
.IP
The root file will contain a table of contents
that directs the reader to the section files, and
each of those begins and ends with a navigation
command area that allows moving one to three
sections in either direction, as well as back up
to the root file.
.IP
This option permits large manual page files to be
split into smaller parts that load faster over the
World-Wide Web, although with the possibly
significant disadvantage that the reader can no
longer search the entire document with a single
command.
.\"=====================================================================
.SH "HTML GRAMMAR LEVELS"
The level 3 grammar has expired; some of its
features, particularly the support for markup of
mathematics, will appear in a future HTML grammar
level.
.PP
The version 3.2 grammar is a stopgap, which,
despite its higher number, lies approximately
between 2 and 3 in features.  It was released on
November 5, 1996, at
\&\fChttp://www.w3.org/pub/WWW/\fP in order to
provide a stable grammar toward which WWW browser
developers could work.
.PP
The next version of HTML, code-named
.IR Cougar ,
is under development, and will become version 4.0
when it is finally released.  The first draft
public release was on 8 July 1997, and that was
followed by a proposed recommended version on 7
November 1997.
.PP
There are only four potential differences in the
output of
.B man2html
for these grammar levels:
.RS
.TP \w'\(bu'u+3n
\(bu
The output \fC<!DOCTYPE HTML PUBLIC ".\|.\|.">\fP
declaration depends on the grammar level.
.TP
\(bu
At version 3 and above, the SGML entity
.I "&nbsp;"
can be used for non-breakable space instead of the
less obvious numeric entity
.I "&#160;"
which is required by the level 2 grammar.
.TP
\(bu
At versions 3 and 3.2, the SGML entity
.IR "&quot;" ,
representing a quotation mark, must be replaced by
a numeric entity,
.IR "&#34;" ,
because of an unfortunate error of omission in the
grammars.
.TP
\(bu
At version 3.2 and higher, the output HTML will
use
.I "<CENTER> \|.\|.\|. </CENTER>"
directives to support centered text.  At earlier
grammar levels, centering requests are ignored,
but the request is preserved in a comment, and
lines are still broken as they would be when
centered.
.IP
Centering is exceedingly rare in manual page files
(it is completely absent from
.I all
of Sun's standard manual pages), so the default
level 2 grammar should almost always be
sufficient.
.RE
.\"=====================================================================
.SH "SEE ALSO"
.BR amaya (1),
.BR arena (1),
.BR chimera (1),
.BR grail (1),
.BR hotjava (1),
.BR html-check (1),
.BR html-ncheck (1),
.BR html-norm (1),
.BR html-pretty (1),
.BR html-spam (1),
.BR html2latex (1),
.BR htmlchek (1),
.BR jde (1),
.BR latex2html (1),
.BR lynx (1),
.BR netscape (1),
.BR nsgmls (1),
.BR panorama (1),
.BR rman (1),
.BR RosettaMan (1),
.BR rtf2html (1),
.BR sgmlnorm (1),
.BR sgmls (1),
.BR spam (1),
.BR spent (1),
.BR texi2html (1),
.BR xmosaic (1).
.\"=====================================================================
.SH AUTHOR
.nf
Nelson H. F. Beebe, Ph.D.
Center for Scientific Computing
University of Utah
Department of Mathematics, 322 INSCC
155 S 1400 E RM 233
Salt Lake City, UT 84112-0090
USA
Tel: +1 801 581 5254
FAX: +1 801 585 1640, +1 801 581 4148
Email: \fCbeebe@math.utah.edu\fP, \fCbeebe@acm.org\fP, \fCbeebe@ieee.org\fP (Internet)
WWW URL: \fChttp://www.math.utah.edu/~beebe\fP
.fi
.\"=====================================================================
.SH AVAILABILITY
.B man2html
is freely available; its master distribution can
be found at
.PP
.RS
.nf
\&\fCftp://ftp.math.utah.edu/pub/sgml/\fP
.fi
.RE
.PP
in the file
.I man2html-x.yy.tar.gz
where
.I x.yy
is the current version.  Other distribution
formats are usually available in the same
location.  Several other SGML and HTML tools are
available in that same directory.
.PP
That site is mirrored to several other Internet
archives, so you may also be able to find it
elsewhere on the Internet; try searching for the
string
.I man2html
at one or more of the popular Web search sites,
such as
.PP
.RS
.nf
\&\fChttp://altavista.digital.com/
http://www.hotbot.com/
http://www.stpt.com/
http://www.yahoo.com/\fP
.fi
.RE
.\"==============================[The End]==============================
